#import "Swrve.h"
#import "SwrveBaseCampaign.h"
#import "SwrveCampaign.h"
#import "SwrvePrivateBaseCampaign.h"
#import "SwrveMessage.h"
#import "SwrveButton.h"
#import "SwrveImage.h"

@implementation SwrveCampaign

@synthesize messages;

-(id)initAtTime:(NSDate*)time fromJSON:(NSDictionary *)dict withAssetsQueue:(NSMutableSet*)assetsQueue forController:(SwrveMessageController*)controller
{
    id instance = [super initAtTime:time fromJSON:dict withAssetsQueue:assetsQueue forController:controller];
    NSMutableArray* loadedMessages = [[NSMutableArray alloc] init];
    NSArray* campaign_messages = [dict objectForKey:@"messages"];
    for (NSDictionary* messageDict in campaign_messages)
    {
        SwrveMessage* message = [SwrveMessage fromJSON:messageDict forCampaign:self forController:controller];
        for (SwrveMessageFormat* format in message.formats)
        {
            // Add all images to the download queue
            for (SwrveButton* button in format.buttons)
            {
                [assetsQueue addObject:button.image];
            }
            
            for (SwrveImage* image in format.images)
            {
                [assetsQueue addObject:image.file];
            }
        }
        [loadedMessages addObject:message];
    }
    self.messages = [[NSArray alloc] initWithArray:loadedMessages];
    return instance;
}

-(void)messageWasShownToUser:(SwrveMessage *)message
{
    [self messageWasShownToUser:message at:[NSDate date]];
}

-(void)messageWasShownToUser:(SwrveMessage*)message at:(NSDate*)timeShown
{
    #pragma unused(message)
    [self incrementImpressions];
    [self setMessageMinDelayThrottle:timeShown];
    
    if (![self randomOrder])
    {
        NSUInteger count = [[self messages] count];
        NSUInteger nextMessage = ([self next] + 1) % count;
        DebugLog(@"Round Robin message in campaign %ld is %ld (next will be %ld)", (unsigned long)[self ID], (unsigned long)[self next], (unsigned long)nextMessage);
        [self setNext:nextMessage];
    }
}

-(void)messageDismissed:(NSDate *)timeDismissed
{
    [self setMessageMinDelayThrottle:timeDismissed];
}

static SwrveMessage* firstFormatFrom(NSArray* messages, NSSet* assets)
{
    // Return the first fully downloaded format
    for (SwrveMessage* message in messages) {
        if ([message areDownloaded:assets]){
            return message;
        }
    }
    return nil;
}

/**
 * Quick check to see if this campaign might have messages matching this event trigger
 * This is used to decide if the campaign is a valid candidate for automatically showing at session start
 */
-(BOOL)hasMessageForEvent:(NSString*)event
{
    return [self triggers] != nil && [[self triggers] containsObject:[event lowercaseString]];
}

-(SwrveMessage*)getMessageForEvent:(NSString*)event
                        withAssets:(NSSet*)assets
                            atTime:(NSDate*)time

{
    return [self getMessageForEvent:event withAssets:assets atTime:time withReasons:nil];
}


-(SwrveMessage*)getMessageForEvent:(NSString*)event
                        withAssets:(NSSet*)assets
                            atTime:(NSDate*)time
                       withReasons:(NSMutableDictionary*)campaignReasons
{
    if (![self hasMessageForEvent:event]){
        DebugLog(@"There is no trigger in %ld that matches %@", (long)self.ID, event);
        return nil;
    }

    if ([[self messages] count] == 0)
    {
        [self logAndAddReason:[NSString stringWithFormat:@"No messages in campaign %ld", (long)self.ID] withReasons:campaignReasons];
        return nil;
    }

    if (![self checkCampaignRulesForEvent:event atTime:time withReasons:campaignReasons]) {
        return nil;
    }

    SwrveMessage* message = nil;
    if (self.randomOrder)
    {
        DebugLog(@"Random Message in %ld", (long)self.ID);
        NSArray* shuffled = [SwrveMessageController shuffled:self.messages];
        message = firstFormatFrom(shuffled, assets);
    }

    if (message == nil)
    {
        message = [self.messages objectAtIndex:(NSUInteger)self.next];
    }
    
    if ([message areDownloaded:assets]) {
        DebugLog(@"%@ matches a trigger in %ld", event, (long)self.ID);
        return message;
    }
    
    [self logAndAddReason:[NSString stringWithFormat:@"Campaign %ld hasn't finished downloading", (long)self.ID] withReasons:campaignReasons];
    return nil;
}

-(NSDictionary*)campaignSettings
{
    NSMutableDictionary* settings = [super campaignSettings];
    [settings setValue:[NSNumber numberWithUnsignedInteger:[self next]] forKey:@"next"];
    return [NSDictionary dictionaryWithDictionary:settings];
}

@end
